### Examples of using RNAconTest
library(Biostrings) # required package

# specify the number of supplemental sequences to test
N <- 10*2^(0:1) # maximally 0:9
print(N)

# load an alignment package (for the example)
library(DECIPHER)

# set the working directory to the path on your computer
setwd("~/Desktop/Alignment/RNABenchmarks/RNAconTest_v1/")

# source the scoring functions
source("./Score.R")

# get all reference sets
ls <- list.files("./ref_sequences/")

# initialize the output array
results <- array(NA_real_,
	dim=c(length(ls), length(N), 2),
	dimnames=list(ls, N, c("ali_score", "ss_score")))

pBar <- txtProgressBar(style=3)
for (i in seq_along(ls)) { # for each reference set
	# load the reference sequences
	ref <- readRNAStringSet(paste0("./ref_sequences/", ls[i]))
	
	# load the reference structures
	struct <- readLines(paste0("./ref_pairings/", ls[i]))
	struct <- setNames(struct[seq(2, length(struct), 2)],
		substring(struct[seq(1, length(struct), 2)], 2))
	struct <- strsplit(struct, "\t", fixed=TRUE)
	struct <- suppressWarnings(lapply(struct, as.integer))
	
	for (j in seq_along(N)) { # for each set of supplemental sequences
		path <- paste0("./sup_sequences/", N[j], "/", ls[i], ".gz")
		if (!file.exists(path))
			break
		
		# read in the supplemental sequences
		seqs <- readDNAStringSet(path)
		
		# combine the reference and supplemental sequences
		seqs <- c(ref, seqs)
		
		# give the sequences unique names
		names(seqs) <- seq_along(seqs)
		
		# write the combined sequences to an input file
		tfin <- tempfile()
		tfout <- tempfile()
		writeXStringSet(seqs, file=tfin)
		
		# perform the alignment with a program
		# typically this would be a system() call
		# e.g., system(paste("myprogram", tfin, ">" tfout))
		seqs_in <- readRNAStringSet(tfin)
		aligned <- AlignSeqs(seqs_in, verbose=FALSE)
		writeXStringSet(aligned, file=tfout)
		
		# read in the aligned output file
		if (!file.exists(tfout)) { # no output file
			unlink(tfin)
			next
		}
		aligned <- readRNAStringSet(tfout)
		
		# select only the reference sequences
		ref_aligned <- aligned[as.character(seq_along(ref))]
		
		# score the alignment
		if (length(ref_aligned) > 1)
			results[i, j, "ali_score"] <- ScoreAlignment(ref_aligned, struct)
		
		# generate a secondary structure prediction
		# oftentimes this is part of the output file
		# but here we will use a stand alone function
		cons <- PredictDBN(aligned, verbose=FALSE)
		
		# score the structure
		results[i, j, "ss_score"] <- ScoreStructure(ref_aligned, struct, cons)
		
		# remove the temporary files
		unlink(c(tfin, tfout))
	}
	
	setTxtProgressBar(pBar, i/length(ls))
}

# display the results
print(results)
